@@ -35,14 +35,12 @@
 #include <limits.h>
 #include <sys/cygwin.h>
 #include <cygwin/version.h>
-#define _WIN32_WINNT 0x0a00
-#define WINVER 0x0a00
 #include <windows.h>
 
 #define LOCALE_ALIAS		"/usr/share/locale/locale.alias"
 #define LOCALE_ALIAS_LINE_LEN	255
 
-void
+static void __attribute__ ((__noreturn__))
 usage ()
 {
   printf (
@@ -78,6 +76,7 @@ usage ()
 "  -h, --help           This text\n"
 "  -V, --version        Print program version and exit\n\n",
   program_invocation_short_name);
+  exit (0);
 }
 
 void
@@ -370,29 +369,6 @@ print_all_locales (int verbose)
 	      add_locale (loc, language, country);
 	    }
 	}
-      /* Check Serbian language for the available territories.  Vista only
-	 had sr_CS.  Only starting with W7 we have the actual sr_RS and sr_ME.
-	 However, they are supported on Vista as well in Cygwin.  So we fake
-	 them here, if they are missing. */
-      if (lang == LANG_SERBIAN)
-	{
-	  int sr_CS_idx = -1;
-	  int sr_RS_idx = -1;
-	  int i;
-
-	  for (i = 0; i < lcnt; ++ i)
-	    if (!strcmp (loc_list[i].loc, "sr_CS"))
-	      sr_CS_idx = i;
-	    else if (!strcmp (loc_list[i].loc, "sr_RS"))
-	      sr_RS_idx = i;
-	  if (sr_CS_idx > 0 && sr_RS_idx == -1)
-	    {
-	      add_locale ("sr_RS@latin", L"Serbian (Latin)", L"Serbia");
-	      add_locale ("sr_RS", L"Serbian (Cyrillic)", L"Serbia");
-	      add_locale ("sr_ME@latin", L"Serbian (Latin)", L"Montenegro");
-	      add_locale ("sr_ME", L"Serbian (Cyrillic)", L"Montenegro");
-	    }
-	}
     }
   /* First sort allows add_locale_alias_locales to bsearch in locales. */
   qsort (locale, loc_num, sizeof (loc_t), compare_locales);
@@ -570,26 +546,26 @@ lc_names_t lc_ctype_names[] =
 {
   { "ctype-class-names", 	 is_string_fake, 0,			 0 },
   { "ctype-map-names",   	 is_string_fake, 2,			 0 },
-  { "ctype-outdigit0_mb",	 is_string,	_NL_CTYPE_OUTDIGITS0_MB, 0 },
-  { "ctype-outdigit1_mb",	 is_string,	_NL_CTYPE_OUTDIGITS1_MB, 0 },
-  { "ctype-outdigit2_mb",	 is_string,	_NL_CTYPE_OUTDIGITS2_MB, 0 },
-  { "ctype-outdigit3_mb",	 is_string,	_NL_CTYPE_OUTDIGITS3_MB, 0 },
-  { "ctype-outdigit4_mb",	 is_string,	_NL_CTYPE_OUTDIGITS4_MB, 0 },
-  { "ctype-outdigit5_mb",	 is_string,	_NL_CTYPE_OUTDIGITS5_MB, 0 },
-  { "ctype-outdigit6_mb",	 is_string,	_NL_CTYPE_OUTDIGITS6_MB, 0 },
-  { "ctype-outdigit7_mb",	 is_string,	_NL_CTYPE_OUTDIGITS7_MB, 0 },
-  { "ctype-outdigit8_mb",	 is_string,	_NL_CTYPE_OUTDIGITS8_MB, 0 },
-  { "ctype-outdigit9_mb",	 is_string,	_NL_CTYPE_OUTDIGITS9_MB, 0 },
-  { "ctype-outdigit0_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGITS0_WC, 0 },
-  { "ctype-outdigit1_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGITS1_WC, 0 },
-  { "ctype-outdigit2_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGITS2_WC, 0 },
-  { "ctype-outdigit3_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGITS3_WC, 0 },
-  { "ctype-outdigit4_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGITS4_WC, 0 },
-  { "ctype-outdigit5_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGITS5_WC, 0 },
-  { "ctype-outdigit6_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGITS6_WC, 0 },
-  { "ctype-outdigit7_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGITS7_WC, 0 },
-  { "ctype-outdigit8_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGITS8_WC, 0 },
-  { "ctype-outdigit9_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGITS9_WC, 0 },
+  { "ctype-outdigit0_mb",	 is_string,	_NL_CTYPE_OUTDIGIT0_MB,  0 },
+  { "ctype-outdigit1_mb",	 is_string,	_NL_CTYPE_OUTDIGIT1_MB,  0 },
+  { "ctype-outdigit2_mb",	 is_string,	_NL_CTYPE_OUTDIGIT2_MB,  0 },
+  { "ctype-outdigit3_mb",	 is_string,	_NL_CTYPE_OUTDIGIT3_MB,  0 },
+  { "ctype-outdigit4_mb",	 is_string,	_NL_CTYPE_OUTDIGIT4_MB,  0 },
+  { "ctype-outdigit5_mb",	 is_string,	_NL_CTYPE_OUTDIGIT5_MB,  0 },
+  { "ctype-outdigit6_mb",	 is_string,	_NL_CTYPE_OUTDIGIT6_MB,  0 },
+  { "ctype-outdigit7_mb",	 is_string,	_NL_CTYPE_OUTDIGIT7_MB,  0 },
+  { "ctype-outdigit8_mb",	 is_string,	_NL_CTYPE_OUTDIGIT8_MB,  0 },
+  { "ctype-outdigit9_mb",	 is_string,	_NL_CTYPE_OUTDIGIT9_MB,  0 },
+  { "ctype-outdigit0_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGIT0_WC,  0 },
+  { "ctype-outdigit1_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGIT1_WC,  0 },
+  { "ctype-outdigit2_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGIT2_WC,  0 },
+  { "ctype-outdigit3_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGIT3_WC,  0 },
+  { "ctype-outdigit4_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGIT4_WC,  0 },
+  { "ctype-outdigit5_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGIT5_WC,  0 },
+  { "ctype-outdigit6_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGIT6_WC,  0 },
+  { "ctype-outdigit7_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGIT7_WC,  0 },
+  { "ctype-outdigit8_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGIT8_WC,  0 },
+  { "ctype-outdigit9_wc",	 is_wchar, 	_NL_CTYPE_OUTDIGIT9_WC,  0 },
   { "charmap",			 is_string,	CODESET,		 0 },
   { "ctype-mb-cur-max",		 is_int,	_NL_CTYPE_MB_CUR_MAX,	 0 },
   { NULL, 			 is_end,	0,		 	 0 }
@@ -811,7 +787,6 @@ main (int argc, char **argv)
 	break;
       case 'h':
 	usage ();
-	return 0;
       case 'V':
 	print_version ();
 	return 0;
